@@ -1,6 +1,6 @@
 /*
   Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-  2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Free Software
+  2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015 Free Software
   Foundation, Inc.
 
   This file is part of GNU Inetutils.
@@ -49,7 +49,8 @@
 
 #include <config.h>
 
-#if defined KRB4 || defined SHISHI
+#if defined KRB5 || defined SHISHI
+
 # include <sys/types.h>
 # ifdef ENCRYPTION
 #  include <sys/socket.h>
@@ -57,14 +58,13 @@
 
 # include <netinet/in.h>
 
-# ifdef KERBEROS
-#  ifdef HAVE_KERBEROSIV_DES_H
-#   include <kerberosIV/des.h>
+# ifdef KRB5
+#  ifdef HAVE_KRB5_H
+#   include <krb5.h>
 #  endif
-#  ifdef HAVE_KERBEROSIV_KRB_H
-#   include <kerberosIV/krb.h>
-#  endif
-# elif defined(SHISHI)
+#  include "kerberos5_def.h"
+
+# elif defined(SHISHI) /* ! KRB5 */
 #  include <shishi.h>
 #  include "shishi_def.h"
 #  ifdef HAVE_GETPWUID_R
@@ -72,7 +72,7 @@
 #   include <unistd.h>
 #   include <pwd.h>
 #  endif /* HAVE_GETPWUID_R */
-# endif /* SHISHI */
+# endif /* SHISHI && !KRB5 */
 
 # include <stdio.h>
 
@@ -80,14 +80,14 @@
 
 # if defined SHISHI
 int kcmd (Shishi **, int *, char **, unsigned short, char *, char **,
-	  char *, int *, char *, char *, Shishi_key **,
+	  char *, int *, char *, const char *, Shishi_key **,
 	  struct sockaddr_storage *, struct sockaddr_storage *,
 	  long, int);
-# else
-int kcmd (int *, char **, unsigned short, char *, char *, char *, int *,
-	  KTEXT, char *, char *, CREDENTIALS *, Key_schedule,
-	  MSG_DAT *, struct sockaddr_in *, struct sockaddr_in *, long);
-# endif
+# else /* KRB5 && !SHISHI */
+int kcmd (krb5_context *, int *, char **, unsigned short, char *, char **,
+	  char *, int *, char *, const char *, krb5_keyblock **,
+	  struct sockaddr_in *, struct sockaddr_in *, long);
+# endif /* !SHISHI */
 
 /*
  * krcmd: simplified version of Athena's "kcmd"
@@ -103,8 +103,8 @@ static struct passwd pwstor, *pwd;
 #  endif /* HAVE_GETPWUID_R */
 
 int
-krcmd (Shishi ** h, char **ahost, unsigned short rport, char **remuser, char *cmd,
-       int *fd2p, char *realm, int af)
+krcmd (Shishi ** h, char **ahost, unsigned short rport, char **remuser,
+       char *cmd, int *fd2p, const char *realm, int af)
 {
   int sock = -1, err = 0;
   long authopts = 0L;
@@ -146,40 +146,44 @@ krcmd (Shishi ** h, char **ahost, unsigned short rport, char **remuser, char *cm
   return (sock);
 }
 
-# elif defined(KERBEROS)
+# elif defined(KRB5) /* !SHISHI */
 int
-krcmd (char **ahost, unsigned short rport, char *remuser, char *cmd, int *fd2p,
-       char *realm)
+krcmd (krb5_context *ctx, char **ahost, unsigned short rport,
+       char **remuser, char *cmd, int *fd2p, const char *realm)
 {
-  int sock = -1, err = 0;
-  KTEXT_ST ticket;
+  int sock = -1;
+  krb5_error_code err = 0;
   long authopts = 0L;
 
-  err = kcmd (&sock, ahost, rport, NULL,	/* locuser not used */
-	      remuser, cmd, fd2p, &ticket, SERVICE_NAME, realm, NULL,	/* credentials not used */
-	      (bit_64 *) NULL,	/* key schedule not used */
-	      (MSG_DAT *) NULL,	/* MSG_DAT not used */
+  err = kcmd (ctx, &sock, ahost, rport,
+	      NULL,	/* locuser not used */
+	      remuser, cmd, fd2p,
+	      SERVICE_NAME, realm,
+	      (krb5_keyblock **) NULL,		/* key not used */
 	      (struct sockaddr_in *) NULL,	/* local addr not used */
 	      (struct sockaddr_in *) NULL,	/* foreign addr not used */
 	      authopts);
 
-  if (err > KSUCCESS && err < MAX_KRB_ERRORS)
+  if (err > 0)
     {
-      fprintf (stderr, "krcmd: %s\n", krb_err_txt[err]);
+      const char *text = krb5_get_error_message (*ctx, err);
+
+      fprintf (stderr, "krcmd: %s\n", text);
+      krb5_free_error_message (*ctx, text);
       return (-1);
     }
   if (err < 0)
     return (-1);
   return (sock);
 }
-# endif
+# endif /* KRB5 && !SHISHI */
 
 # ifdef ENCRYPTION
 
 #  if defined SHISHI
 int
 krcmd_mutual (Shishi ** h, char **ahost, unsigned short rport, char **remuser,
-	      char *cmd, int *fd2p, char *realm, Shishi_key ** key, int af)
+	      char *cmd, int *fd2p, const char *realm, Shishi_key ** key, int af)
 {
   int sock = -1, err = 0;
   struct sockaddr_storage laddr, faddr;
@@ -206,7 +210,8 @@ krcmd_mutual (Shishi ** h, char **ahost, unsigned short rport, char **remuser,
 	      NULL,		/* locuser not used */
 #   endif
 	      remuser, cmd, fd2p,
-	      SERVICE_NAME, realm, key,	/* filled in */
+	      SERVICE_NAME, realm,
+	      key,		/* filled in */
 	      &laddr,		/* filled in */
 	      &faddr,		/* filled in */
 	      authopts, af);
@@ -223,28 +228,32 @@ krcmd_mutual (Shishi ** h, char **ahost, unsigned short rport, char **remuser,
   return (sock);
 }
 
-#  elif defined(KERBEROS)
+#  elif defined(KRB5) /* !SHISHI */
 int
-krcmd_mutual (char **ahost, unsigned short rport, char *remuser, char *cmd,
-	      int *fd2p, char *realm, CREDENTIALS * cred, Key_schedule sched)
+krcmd_mutual (krb5_context *ctx, char **ahost, unsigned short rport,
+	      char **remuser, char *cmd, int *fd2p, const char *realm,
+	      krb5_keyblock **key)
 {
-  int sock, err;
-  KTEXT_ST ticket;
-  MSG_DAT msg_dat;
+  int sock;
+  krb5_error_code err = 0;
   struct sockaddr_in laddr, faddr;
-  long authopts = KOPT_DO_MUTUAL;
+  long authopts = AP_OPTS_MUTUAL_REQUIRED | AP_OPTS_USE_SUBKEY;
 
-  err = kcmd (&sock, ahost, rport, NULL,	/* locuser not used */
-	      remuser, cmd, fd2p, &ticket, SERVICE_NAME, realm, cred,	/* filled in */
-	      sched,		/* filled in */
-	      &msg_dat,		/* filled in */
+  err = kcmd (ctx, &sock, ahost, rport,
+	      NULL,		/* locuser not used */
+	      remuser, cmd, fd2p,
+	      SERVICE_NAME, realm,
+	      key,		/* filled in */
 	      &laddr,		/* filled in */
 	      &faddr,		/* filled in */
 	      authopts);
 
-  if (err > KSUCCESS && err < MAX_KRB_ERRORS)
+  if (err > 0)
     {
-      fprintf (stderr, "krcmd_mutual: %s\n", krb_err_txt[err]);
+      const char *text = krb5_get_error_message (*ctx, err);
+
+      fprintf (stderr, "krcmd_mutual: %s\n", text);
+      krb5_free_error_message (*ctx, text);
       return (-1);
     }
 
@@ -252,6 +261,6 @@ krcmd_mutual (char **ahost, unsigned short rport, char *remuser, char *cmd,
     return (-1);
   return (sock);
 }
-#  endif /* CRYPT */
-# endif	/* KERBEROS */
-#endif /* KERBEROS */
+#  endif /* KRB5 && !SHISHI */
+# endif /* ENCRYPTION  */
+#endif /* KRB5 || SHISHI */
