@@ -1,6 +1,6 @@
 #!/bin/sh
 
-# Copyright (C) 2011, 2012, 2013 Free Software Foundation, Inc.
+# Copyright (C) 2011, 2012, 2013, 2014, 2015 Free Software Foundation, Inc.
 #
 # This file is part of GNU Inetutils.
 #
@@ -36,6 +36,7 @@ The following environment variables are used:
 VERBOSE		Be verbose, if set.
 FORMAT		Test only these output formats.  A list of
 		formats is excepted.
+TARGET		Loopback address; defaults to 127.0.0.1.
 
 HERE
     exit 0
@@ -48,11 +49,12 @@ fi
 
 . ./tools.sh
 
-# Executable uder test.
+TARGET=${TARGET:-127.0.0.1}
+
+# Executable under test.
 #
 IFCONFIG=${IFCONFIG:-../ifconfig/ifconfig$EXEEXT}
 
-
 if test ! -x "$IFCONFIG"; then
     echo >&2 "Missing executable '$IFCONFIG'.  Skipping test."
     exit 77
@@ -85,21 +87,47 @@ if test -z "$LO"; then
     exit 1
 fi
 
+target=`echo $TARGET | $SED -e 's,\.,\\\\&,g'`
+
 find_lo_addr () {
    $IFCONFIG ${1+--format=$1} -i $LO | \
-   eval $GREP '"inet .*127\.0\.0\.1"' $bucket 2>/dev/null
+   eval $GREP "'inet .*$target'" $bucket 2>/dev/null
 }
 
 errno=0
 
+# Check for loopback address in all formats displaying the
+# standard address $TARGET, commonly 127.0.0.1.
+#
 for fmt in ${FORMAT:-gnu gnu-one-entry net-tools osf unix}; do
     $silence echo "Checking format $fmt."
     find_lo_addr $fmt || { errno=1; echo >&2 "Failed with format '$fmt'."; }
 done
 
+# Check that all listed adapters are responding affirmatively
+# to all formats, but discard all output.
+#
+for fmt in check default gnu gnu-one-entry netstat net-tools osf unix
+do
+    for iface in $IF_LIST; do
+	$IFCONFIG --format=$fmt -i $iface >/dev/null ||
+	    { errno=1;
+	      echo >&2 "Failed with format '$fmt', adapter '$iface'."
+	    }
+    done
+done
+
+# Check that short format and full printout succeed; discard output.
+#
+$IFCONFIG --short >/dev/null ||
+    { errno=1; echo >&2 'Failed with short format.'; }
+
+$IFCONFIG --all >/dev/null ||
+    { errno=1; echo >&2 'Failed while listing all.'; }
+
 # Informational check whether the legacy form use
 # is implemented.  No error produced, only message.
-
+#
 if $IFCONFIG $LO >/dev/null 2>&1; then
     :
 else
